//
// AccountContext.java
//
// Copyright (c) 1997 by Cornell University
// All rights reserved.
//
// Permission is hereby granted, without written agreement and without
// license or royalty fee, to use, copy, modify, and distribute this
// software and its documentation for any purpose, provided that the
// above copyright notice and the following two paragraphs appear in
// all copies of this software
//
// IN NO EVENT SHALL CORNELL UNIVERSITY BE LIABLE TO ANY PARTY FOR
// DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES
// ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN
// IF CORNELL UNIVERSITY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH
// DAMAGE.
//
// CORNELL UNIVERSITY SPECIFICALLY DISCLAIMS ANY WARRANTIES, INCLUDING,
// BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
// FITNESS FOR A PARTICULAR PURPOSE.  THE SOFTWARE PROVIDED HEREUNDER IS
// ON AN "AS IS" BASIS, AND CORNELL UNIVERSITY HAS NO OBLIGATION TO
// PROVIDE MAINTAINANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
//

package SLK;

import java.util.*;

public final class AccountContext
{
	int hash;

	Account account;
	// either parent == null, accountSwitch == null, description != null
	// or     parent != null, accountSwitch != null, description == null
	AccountContext parent;
	AccountSwitch accountSwitch;
	String description; // info to help understand accounting printouts

	double time;
	long n;

	static Hashtable accountContexts = new Hashtable();

	public String toString()
	{
		if (parent != null) {
		    return parent + " calls <" + accountSwitch + "(account:" + account + "):" +
		        n + " times(" + time + " seconds)>";
		}
		else {
		    return "<" + description + "(account:" + account + "):" + n + " times(" +
		        time + " seconds)>";
		}
	}

	// this probably shouldn't be public, but for now its ok:
	public static String getStats()
	{
		String s = "";
		Enumeration e = accountContexts.elements();
		while (e.hasMoreElements()) {
			s += e.nextElement() + "\n\n";
		}
		return s;
	}

	public AccountContext(String description, Account account)
	{
		this.parent = null;
		if (account == null) throw new NullPointerException();
		this.account = account;
		if (description == null) throw new NullPointerException();
		this.description = "\"" + description + "\"";
		hash = super.hashCode();
	}

	static AccountContext getAccountContext(AccountContext parent, AccountSwitch accountSwitch, Account account)
	{
		// this could be more efficient, but...
		AccountContext modelAccountContext = new AccountContext(parent, accountSwitch, account);
		AccountContext accountContext = (AccountContext) accountContexts.get(modelAccountContext);

		if (accountContext == null) {
			accountContext = modelAccountContext;
			accountContexts.put(accountContext, accountContext);
		}

		return accountContext;
	}

	AccountContext(AccountContext parent, AccountSwitch accountSwitch, Account account)
	{
		this.parent = parent;
		this.accountSwitch = accountSwitch;
		this.account = account;
		hash = parent.hash * 37 + account.hash;
	}

	public int hashCode()
	{
		return hash;
	}

	public boolean equals(Object obj)
	{
		if (obj instanceof AccountContext) {
			AccountContext target = (AccountContext) obj;
			if (parent == null) {
			    return target == this;
			}
			else {
			    return  account == target.account &&
			            accountSwitch == target.accountSwitch &&
			            parent.equals(target.parent);
			}
		}
		else return false;
	}
}