//
// AccountSwitch.java
//
// Copyright (c) 1997 by Cornell University
// All rights reserved.
//
// Permission is hereby granted, without written agreement and without
// license or royalty fee, to use, copy, modify, and distribute this
// software and its documentation for any purpose, provided that the
// above copyright notice and the following two paragraphs appear in
// all copies of this software
//
// IN NO EVENT SHALL CORNELL UNIVERSITY BE LIABLE TO ANY PARTY FOR
// DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES
// ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN
// IF CORNELL UNIVERSITY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH
// DAMAGE.
//
// CORNELL UNIVERSITY SPECIFICALLY DISCLAIMS ANY WARRANTIES, INCLUDING,
// BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
// FITNESS FOR A PARTICULAR PURPOSE.  THE SOFTWARE PROVIDED HEREUNDER IS
// ON AN "AS IS" BASIS, AND CORNELL UNIVERSITY HAS NO OBLIGATION TO
// PROVIDE MAINTAINANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
//

package SLK;

import java.util.*;

public abstract class AccountSwitch
{
	Account calleeAccount; // this may be null
	String description; // info to help understand accounting printouts
	int descriptionHash;

	public String toString() {return description;}

	// this method makes the callee pay (assuming the callee is willing):
	public Object cross(AccountContext callerContext, Object arg)
		throws Exception
	{
		if(calleeAccount == null) throw new SecurityException("callee unwilling to pay");
		Object retVal;
		long start100Nanos = ThreadTimer.thread100Nanos();
		AccountContext calleeContext = AccountContext.getAccountContext(callerContext, this, calleeAccount);

		try
		{
			retVal = meth(calleeContext, arg);
		}
		finally
		{
			long end100Nanos = ThreadTimer.thread100Nanos();
			double diff = (end100Nanos - start100Nanos) * 0.0000001;
			synchronized(calleeContext)
			{
				calleeContext.time += diff;
				calleeContext.n++;
			}
		}
		return retVal;
	}

	// this method makes the caller pay:
	public Object cross(AccountContext callerContext, Object arg, Account callerAccount)
		throws Exception
	{
		Object retVal;
		long start100Nanos = ThreadTimer.thread100Nanos();
		AccountContext calleeContext = AccountContext.getAccountContext(callerContext, this, callerAccount);

		try
		{
			retVal = meth(calleeContext, arg);
		}
		finally
		{
			long end100Nanos = ThreadTimer.thread100Nanos();
			double diff = (end100Nanos - start100Nanos) * 0.0000001;
			synchronized(calleeContext)
			{
				calleeContext.time += diff;
				calleeContext.n++;
			}
		}
		return retVal;
	}

	protected AccountSwitch(String description)
	{
		this(description, null);
	}

	protected AccountSwitch(String description, Account calleeAccount)
	{
		this.description = "\"" + description + "\"";
		this.calleeAccount = calleeAccount;
		this.descriptionHash = description.hashCode();
	}

	protected abstract Object meth(AccountContext calleeContext, Object arg) throws Exception;
}


