//
// SafeBox.java
//
// Copyright (c) 1997 by Cornell University
// All rights reserved.
//
// Permission is hereby granted, without written agreement and without
// license or royalty fee, to use, copy, modify, and distribute this
// software and its documentation for any purpose, provided that the
// above copyright notice and the following two paragraphs appear in
// all copies of this software
//
// IN NO EVENT SHALL CORNELL UNIVERSITY BE LIABLE TO ANY PARTY FOR
// DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES
// ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN
// IF CORNELL UNIVERSITY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH
// DAMAGE.
//
// CORNELL UNIVERSITY SPECIFICALLY DISCLAIMS ANY WARRANTIES, INCLUDING,
// BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
// FITNESS FOR A PARTICULAR PURPOSE.  THE SOFTWARE PROVIDED HEREUNDER IS
// ON AN "AS IS" BASIS, AND CORNELL UNIVERSITY HAS NO OBLIGATION TO
// PROVIDE MAINTAINANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
//

package SLK;

/**
 * A SafeBox class stores an object and locks it with a public key such that
 * only the holder of the corresponding private key can access the stored
 * object.
 */

public final class SafeBox
{
    private Object o;
    private Key pubKey;

    /**
     * Create a SafeBox.
     * @params o The object to be safe guarded.
     * @params publicKey Public key that "locks" the safe.
     */
    public SafeBox(Object o, Key publicKey)
    {
        this.o = o;
        pubKey = publicKey;
    }

    /**
     * Unlock the Safebox.
     * @params privateKey The private key to be matched with the public key
     * that locks the SafeBox.
     * @return The object stored in the SafeBox or null.
    */
    public Object unlock(Key privateKey)
    {
        return pubKey.match(privateKey) ? o : null;
    }
}