//
// ThreadRights.java
//
// Copyright (c) 1997 by Cornell University
// All rights reserved.
//
// Permission is hereby granted, without written agreement and without
// license or royalty fee, to use, copy, modify, and distribute this
// software and its documentation for any purpose, provided that the
// above copyright notice and the following two paragraphs appear in
// all copies of this software
//
// IN NO EVENT SHALL CORNELL UNIVERSITY BE LIABLE TO ANY PARTY FOR
// DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES
// ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN
// IF CORNELL UNIVERSITY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH
// DAMAGE.
//
// CORNELL UNIVERSITY SPECIFICALLY DISCLAIMS ANY WARRANTIES, INCLUDING,
// BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
// FITNESS FOR A PARTICULAR PURPOSE.  THE SOFTWARE PROVIDED HEREUNDER IS
// ON AN "AS IS" BASIS, AND CORNELL UNIVERSITY HAS NO OBLIGATION TO
// PROVIDE MAINTAINANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
//

package SLK;

/**
 * ThreadRights represent the right to perform some action on the current
 * thread.
 */

public class ThreadRights
{
	SlkThread thread = null;

	public ThreadRights()
	{
	}

	synchronized void setThread(SlkThread thread)
		throws SecurityException
	{
		if(this.thread != null) throw new SecurityException();
		this.thread = thread;
	}

	public final synchronized void stop()
		throws SecurityException
	{
		SlkThread currentThread = (SlkThread) Thread.currentThread();

		synchronized(currentThread.switchLock)
		{
			try
			{
				if(this == currentThread.currentRights)
				{
					currentThread.targetThread = thread;
				}

				thread.stop();
			}
			finally
			{
				currentThread.targetThread = null;
			}
		}
	}

	public final synchronized void stop(Throwable o)
	throws SecurityException
	{
		SlkThread currentThread = (SlkThread) Thread.currentThread();

		synchronized(currentThread.switchLock)
		{
			try
			{
				if(this == currentThread.currentRights)
				{
					currentThread.targetThread = thread;
				}

	    		thread.stop(o);
			}
			finally
			{
				currentThread.targetThread = null;
			}
		}
	}

	public final void suspend()
	throws SecurityException
	{
		SlkThread currentThread = (SlkThread) Thread.currentThread();

		synchronized(currentThread.switchLock)
		{
			try
			{
				if(this == currentThread.currentRights)
				{
					currentThread.targetThread = thread;
				}

				thread.suspend();
			}
			finally
			{
				currentThread.targetThread = null;
			}
		}
	}

	public final void resume()
	throws SecurityException
	{
		SlkThread currentThread = (SlkThread) Thread.currentThread();

		synchronized(currentThread.switchLock)
		{
			try
			{
				if(this == currentThread.currentRights)
				{
					currentThread.targetThread = thread;
				}

				thread.resume();
			}
			finally
			{
				currentThread.targetThread = null;
			}
		}
	}

    public final void setName(String Name)
    throws SecurityException
    {
		SlkThread currentThread = (SlkThread) Thread.currentThread();

		synchronized(currentThread.switchLock)
		{
			try
			{
				if(this == currentThread.currentRights)
				{
					currentThread.targetThread = thread;
				}

				thread.setName(Name);
			}
			finally
			{
				currentThread.targetThread = null;
			}
		}
	}

	public final void setPriority(int newPriority)
	throws SecurityException
	{
	    // Hack: in order not to have to change all the Thread
	    // stuff in httpd, we comment out the code below...

		SlkThread currentThread = (SlkThread) Thread.currentThread();

		synchronized(currentThread.switchLock)
		{
			try
			{
				if(this == currentThread.currentRights)
				{
					currentThread.targetThread = thread;
				}

				thread.setPriority(newPriority);
			}
			finally
			{
				currentThread.targetThread = null;
			}
		}

	}

	public final void setDaemon(boolean on)
	throws SecurityException
	{
		SlkThread currentThread = (SlkThread) Thread.currentThread();

		synchronized(currentThread.switchLock)
		{
			try
			{
				if(this == currentThread.currentRights)
				{
					currentThread.targetThread = thread;
				}

				thread.setDaemon(on);
			}
			finally
			{
				currentThread.targetThread = null;
			}
		}
	}
}